@@ -27,15 +27,15 @@ import (
 	"strings"
 	"syscall"
 
-	"github.com/appc/cni/pkg/ip"
-	cnitypes "github.com/appc/cni/pkg/types"
-	cniutils "github.com/appc/cni/pkg/utils"
 	"github.com/appc/spec/schema/types"
+	"github.com/containernetworking/cni/pkg/ip"
+	cnitypes "github.com/containernetworking/cni/pkg/types"
+	cniutils "github.com/containernetworking/cni/pkg/utils"
+	cnisysctl "github.com/containernetworking/cni/pkg/utils/sysctl"
 	"github.com/hashicorp/errwrap"
 	"github.com/vishvananda/netlink"
 
 	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/networking/netinfo"
 	"github.com/coreos/rkt/networking/tuntap"
 )
 
@@ -79,6 +79,10 @@ type MacVTapNetConf struct {
 	Mode   string `json:"mode"`
 }
 
+const (
+	IPv4InterfaceArpProxySysctlTemplate = "net.ipv4.conf.%s.proxy_arp"
+)
+
 // setupTapDevice creates persistent macvtap device
 // and returns a newly created netlink.Link structure
 // using part of pod hash and interface number in interface name
@@ -121,6 +125,15 @@ func setupMacVTapDevice(podID types.UUID, config MacVTapNetConf, interfaceNumber
 	if err := netlink.LinkAdd(link); err != nil {
 		return nil, errwrap.Wrap(errors.New("cannot create macvtap interface"), err)
 	}
+
+	// TODO: duplicate following lines for ipv6 support, when it will be added in other places
+	ipv4SysctlValueName := fmt.Sprintf(IPv4InterfaceArpProxySysctlTemplate, interfaceName)
+	if _, err := cnisysctl.Sysctl(ipv4SysctlValueName, "1"); err != nil {
+		// remove the newly added link and ignore errors, because we already are in a failed state
+		_ = netlink.LinkDel(link)
+		return nil, errwrap.Wrap(fmt.Errorf("failed to set proxy_arp on newly added interface %q", interfaceName), err)
+	}
+
 	if err := netlink.LinkSetUp(link); err != nil {
 		// remove the newly added link and ignore errors, because we already are in a failed state
 		_ = netlink.LinkDel(link)
@@ -144,7 +157,7 @@ func kvmSetupNetAddressing(network *Networking, n activeNet, ifName string) erro
 	output, err := network.execNetPlugin("ADD", &n, ifName)
 	n.conf.Type = original_type
 	if err != nil {
-		return errwrap.Wrap(fmt.Errorf("problem executing network plugin %q (%q)", n.conf.Type, ifName), err)
+		return errwrap.Wrap(fmt.Errorf("problem executing network plugin %q (%q)", n.conf.IPAM.Type, ifName), err)
 	}
 
 	result := cnitypes.Result{}
@@ -397,12 +410,11 @@ func kvmTransformFlannelNetwork(net *activeNet) error {
 		return errwrap.Wrap(errors.New("error in marshaling generated network settings"), err)
 	}
 
+	net.runtime.IP4 = &cnitypes.IPConfig{}
 	*net = activeNet{
 		confBytes: bytes,
 		conf:      &NetConf{},
-		runtime: &netinfo.NetInfo{
-			IP4: &cnitypes.IPConfig{},
-		},
+		runtime:   net.runtime,
 	}
 	net.conf.Name = n.Name
 	net.conf.Type = n.Delegate["type"].(string)
@@ -552,10 +564,13 @@ func kvmSetup(podRoot string, podID types.UUID, fps []ForwardedPort, netList com
 		}
 		network.nets[i] = n
 	}
-	err := network.forwardPorts(fps, network.GetDefaultIP())
+	podIP, err := network.GetForwardableNetPodIP()
 	if err != nil {
 		return nil, err
 	}
+	if err := network.forwardPorts(fps, podIP); err != nil {
+		return nil, err
+	}
 
 	return &network, nil
 }
@@ -568,6 +583,13 @@ extend Networking struct with methods to clean up kvm specific network configura
 // removing tuntap interface and releasing its ip from IPAM plugin
 func (n *Networking) teardownKvmNets() {
 	for _, an := range n.nets {
+		if an.conf.Type == "flannel" {
+			if err := kvmTransformFlannelNetwork(&an); err != nil {
+				stderr.PrintE("error transforming flannel network", err)
+				continue
+			}
+		}
+
 		switch an.conf.Type {
 		case "ptp", "bridge":
 			// remove tuntap interface
