@@ -28,7 +28,7 @@ import (
 
 	"github.com/coreos/rkt/pkg/multicall"
 	"github.com/coreos/rkt/pkg/sys"
-	"github.com/coreos/rkt/pkg/uid"
+	"github.com/coreos/rkt/pkg/user"
 )
 
 func init() {
@@ -427,7 +427,7 @@ func TestExtractTarOverwrite(t *testing.T) {
 	if !sys.HasChrootCapability() {
 		t.Skipf("chroot capability not available. Disabling test.")
 	}
-	testExtractTarOverwrite(t, extractTarHelper)
+	testExtractTarOverwrite(t, extractTarOverwriteHelper)
 }
 func TestExtractTarOverwriteInsecure(t *testing.T) {
 	testExtractTarOverwrite(t, extractTarInsecureHelper)
@@ -787,12 +787,16 @@ func testExtractTarHardLink(t *testing.T, extractTar func(io.Reader, string) err
 	}
 }
 
+func extractTarOverwriteHelper(rdr io.Reader, target string) error {
+	return ExtractTar(rdr, target, true, user.NewBlankUidRange(), nil)
+}
+
 func extractTarHelper(rdr io.Reader, target string) error {
 	return extractTarHelperPWL(rdr, target, nil)
 }
 
 func extractTarHelperPWL(rdr io.Reader, target string, pwl PathWhitelistMap) error {
-	return ExtractTar(rdr, target, false, uid.NewBlankUidRange(), pwl)
+	return ExtractTar(rdr, target, false, user.NewBlankUidRange(), pwl)
 }
 
 func extractTarInsecureHelper(rdr io.Reader, target string) error {
@@ -800,7 +804,7 @@ func extractTarInsecureHelper(rdr io.Reader, target string) error {
 }
 
 func extractTarInsecureHelperPWL(rdr io.Reader, target string, pwl PathWhitelistMap) error {
-	editor, err := NewUidShiftingFilePermEditor(uid.NewBlankUidRange())
+	editor, err := NewUidShiftingFilePermEditor(user.NewBlankUidRange())
 	if err != nil {
 		return err
 	}
