@@ -20,7 +20,7 @@ package stage0
 // rkt is a reference implementation of the app container specification.
 //
 // Execution on rkt is divided into a number of stages, and the `rkt`
-// binary implements the first stage (stage 0)
+// binary implements the first stage (stage0)
 //
 
 import (
@@ -47,7 +47,7 @@ import (
 	"github.com/coreos/rkt/pkg/label"
 	"github.com/coreos/rkt/pkg/sys"
 	"github.com/coreos/rkt/pkg/tpm"
-	"github.com/coreos/rkt/pkg/uid"
+	"github.com/coreos/rkt/pkg/user"
 	"github.com/coreos/rkt/store"
 	"github.com/coreos/rkt/version"
 	"github.com/hashicorp/errwrap"
@@ -72,11 +72,12 @@ type PrepareConfig struct {
 	Apps               *apps.Apps          // apps to prepare
 	InheritEnv         bool                // inherit parent environment into apps
 	ExplicitEnv        []string            // always set these environment variables for all the apps
+	EnvFromFile        []string            // environment variables loaded from files, set for all the apps
 	Ports              []types.ExposedPort // list of ports that rkt will expose on the host
 	UseOverlay         bool                // prepare pod with overlay fs
 	SkipTreeStoreCheck bool                // skip checking the treestore before rendering
 	PodManifest        string              // use the pod manifest specified by the user, this will ignore flags such as '--volume', '--port', etc.
-	PrivateUsers       *uid.UidRange       // User namespaces
+	PrivateUsers       *user.UidRange      // User namespaces
 }
 
 // configuration parameters needed by Run
@@ -125,20 +126,14 @@ func debug(format string, i ...interface{}) {
 	}
 }
 
-// MergeEnvs amends appEnv setting variables in setEnv before setting anything new from os.Environ if inheritEnv = true
-// setEnv is expected to be in the os.Environ() key=value format
-func MergeEnvs(appEnv *types.Environment, inheritEnv bool, setEnv []string) {
-	for _, ev := range setEnv {
+// mergeEnvs merges environment variables from env into the current appEnv
+// if override is set to true, then variables with the same name will be set to the value in env
+// env is expected to be in the os.Environ() key=value format
+func mergeEnvs(appEnv *types.Environment, env []string, override bool) {
+	for _, ev := range env {
 		pair := strings.SplitN(ev, "=", 2)
-		appEnv.Set(pair[0], pair[1])
-	}
-
-	if inheritEnv {
-		for _, ev := range os.Environ() {
-			pair := strings.SplitN(ev, "=", 2)
-			if _, exists := appEnv.Get(pair[0]); !exists {
-				appEnv.Set(pair[0], pair[1])
-			}
+		if _, exists := appEnv.Get(pair[0]); override || !exists {
+			appEnv.Set(pair[0], pair[1])
 		}
 	}
 }
@@ -248,6 +243,30 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 			ra.App.Isolators = append(ra.App.Isolators, isolator)
 		}
 
+		if app.CapsRetain != nil && app.CapsRemove != nil {
+			return fmt.Errorf("error: cannot use both --cap-retain and --cap-remove on the same image")
+		}
+
+		// Delete existing caps isolators if the user wants to override
+		// them with either --cap-retain or --cap-remove
+		if app.CapsRetain != nil || app.CapsRemove != nil {
+			for i := len(ra.App.Isolators) - 1; i >= 0; i-- {
+				isolator := ra.App.Isolators[i]
+				if _, ok := isolator.Value().(types.LinuxCapabilitiesSet); ok {
+					ra.App.Isolators = append(ra.App.Isolators[:i],
+						ra.App.Isolators[i+1:]...)
+				}
+			}
+		}
+
+		if capsRetain := app.CapsRetain; capsRetain != nil {
+			isolator := capsRetain.AsIsolator()
+			ra.App.Isolators = append(ra.App.Isolators, isolator)
+		} else if capsRemove := app.CapsRemove; capsRemove != nil {
+			isolator := capsRemove.AsIsolator()
+			ra.App.Isolators = append(ra.App.Isolators, isolator)
+		}
+
 		if user := app.User; user != "" {
 			ra.App.User = user
 		}
@@ -256,9 +275,14 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 			ra.App.Group = group
 		}
 
-		if cfg.InheritEnv || len(cfg.ExplicitEnv) > 0 {
-			MergeEnvs(&ra.App.Environment, cfg.InheritEnv, cfg.ExplicitEnv)
+		// loading the environment from the lowest priority to highest
+		if cfg.InheritEnv {
+			// Inherit environment does not override app image environment
+			mergeEnvs(&ra.App.Environment, os.Environ(), false)
 		}
+
+		mergeEnvs(&ra.App.Environment, cfg.EnvFromFile, true)
+		mergeEnvs(&ra.App.Environment, cfg.ExplicitEnv, true)
 		pm.Apps = append(pm.Apps, ra)
 		return nil
 	}); err != nil {
@@ -415,6 +439,12 @@ func addResolvConf(cfg RunConfig, rootfs string) {
 	}
 	content += "\n"
 
+	if err := os.Mkdir(filepath.Join(rootfs, "etc"), defaultRegularDirPerm); err != nil {
+		if !os.IsExist(err) {
+			log.Fatalf("error creating dir %q: %v\n", "/etc", err)
+		}
+	}
+
 	if err := ioutil.WriteFile(filepath.Join(rootfs, "etc/rkt-resolv.conf"), []byte(content), 0644); err != nil {
 		log.Fatalf("error writing /etc/rkt-resolv.conf: %v\n", err)
 	}
@@ -520,7 +550,7 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 		log.Fatalf("error clearing FD_CLOEXEC on lock fd")
 	}
 
-	tpmEvent := fmt.Sprintf("rkt: Rootfs: %s Manifest: %s Stage 1 args: %s", cfg.CommonConfig.RootHash, cfg.CommonConfig.ManifestData, strings.Join(args, " "))
+	tpmEvent := fmt.Sprintf("rkt: Rootfs: %s Manifest: %s Stage1 args: %s", cfg.CommonConfig.RootHash, cfg.CommonConfig.ManifestData, strings.Join(args, " "))
 	// If there's no TPM available or there's a failure for some other
 	// reason, ignore it and continue anyway. Long term we'll want policy
 	// that enforces TPM behaviour, but we don't have any infrastructure
@@ -570,7 +600,7 @@ func prepareAppImage(cfg PrepareConfig, appName types.ACName, img types.Hash, cd
 		if !cfg.SkipTreeStoreCheck {
 			hash, err := cfg.Store.CheckTreeStore(treeStoreID)
 			if err != nil {
-				log.PrintE("warning: tree cache is in a bad state: %v. Rebuilding...", err)
+				log.PrintE("warning: tree cache is in a bad state.  Rebuilding...", err)
 				var err error
 				treeStoreID, hash, err = cfg.Store.RenderTreeStore(img.String(), true)
 				if err != nil {
