@@ -28,11 +28,8 @@ import (
 	"regexp"
 	"strconv"
 	"strings"
-	"syscall"
 
 	"github.com/coreos/rkt/pkg/acl"
-	"github.com/coreos/rkt/pkg/group"
-	"github.com/coreos/rkt/pkg/passwd"
 	stage1commontypes "github.com/coreos/rkt/stage1/common/types"
 
 	"github.com/appc/spec/schema"
@@ -43,7 +40,7 @@ import (
 	"github.com/coreos/rkt/common"
 	"github.com/coreos/rkt/common/cgroup"
 	"github.com/coreos/rkt/pkg/fileutil"
-	"github.com/coreos/rkt/pkg/uid"
+	"github.com/coreos/rkt/pkg/user"
 )
 
 const (
@@ -60,25 +57,6 @@ var (
 		"LOGNAME": "root",
 		"HOME":    "/root",
 	}
-
-	// appDefaultCapabilities defines a restricted set of capabilities given to
-	// apps by default.
-	// See https://github.com/appc/spec/issues/598
-	appDefaultCapabilities, _ = types.NewLinuxCapabilitiesRetainSet(
-		"CAP_AUDIT_WRITE",
-		"CAP_CHOWN",
-		"CAP_DAC_OVERRIDE",
-		"CAP_FSETID",
-		"CAP_FOWNER",
-		"CAP_KILL",
-		"CAP_MKNOD",
-		"CAP_NET_RAW",
-		"CAP_NET_BIND_SERVICE",
-		"CAP_SETUID",
-		"CAP_SETGID",
-		"CAP_SETPCAP",
-		"CAP_SETFCAP",
-		"CAP_SYS_CHROOT")
 )
 
 // execEscape uses Golang's string quoting for ", \, \n, and regex for special cases
@@ -271,7 +249,7 @@ func findHostPort(pm schema.PodManifest, name types.ACName) uint {
 // service files of apps.
 // If there're several apps defining the same UIDs/GIDs, systemd will take care
 // of only generating one /etc/{passwd,group} entry
-func generateSysusers(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uid_ int, gid_ int, uidRange *uid.UidRange) error {
+func generateSysusers(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uid_ int, gid_ int, uidRange *user.UidRange) error {
 	var toShift []string
 
 	app := ra.App
@@ -414,7 +392,7 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 	envFilePathSystemd := EnvFilePathSystemd(p.Root, appName)
 	envFilePathEnterexec := EnvFilePathEnterexec(p.Root, appName)
 
-	uidRange := uid.NewBlankUidRange()
+	uidRange := user.NewBlankUidRange()
 	if err := uidRange.Deserialize([]byte(privateUsers)); err != nil {
 		return err
 	}
@@ -446,10 +424,12 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 		supplementaryGroups = append(supplementaryGroups, strconv.Itoa(g))
 	}
 
-	// TODO: read the RemoveSet as well. See
-	// https://github.com/coreos/rkt/issues/2348#issuecomment-211796840
-	capabilities := append(app.Isolators, appDefaultCapabilities.AsIsolator())
-	capabilitiesStr := GetAppCapabilities(capabilities)
+	capabilitiesStr, err := getAppCapabilities(app.Isolators)
+	if err != nil {
+		return err
+	}
+
+	noNewPrivileges := getAppNoNewPrivileges(app.Isolators)
 
 	execStart := append([]string{binPath}, app.Exec[1:]...)
 	execStartString := quoteExec(execStart)
@@ -460,14 +440,57 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 		unit.NewUnitOption("Service", "Restart", "no"),
 		unit.NewUnitOption("Service", "ExecStart", execStartString),
 		unit.NewUnitOption("Service", "RootDirectory", common.RelAppRootfsPath(appName)),
+		// MountFlags=shared creates a new mount namespace and (as unintuitive
+		// as it might seem) makes sure the mount is slave+shared.
+		unit.NewUnitOption("Service", "MountFlags", "shared"),
 		unit.NewUnitOption("Service", "WorkingDirectory", workDir),
 		unit.NewUnitOption("Service", "EnvironmentFile", RelEnvFilePathSystemd(appName)),
 		unit.NewUnitOption("Service", "User", strconv.Itoa(u)),
 		unit.NewUnitOption("Service", "Group", strconv.Itoa(g)),
 		unit.NewUnitOption("Service", "SupplementaryGroups", strings.Join(supplementaryGroups, " ")),
 		unit.NewUnitOption("Service", "CapabilityBoundingSet", strings.Join(capabilitiesStr, " ")),
+		unit.NewUnitOption("Service", "NoNewPrivileges", strconv.FormatBool(noNewPrivileges)),
+		// This helps working around a race
+		// (https://github.com/systemd/systemd/issues/2913) that causes the
+		// systemd unit name not getting written to the journal if the unit is
+		// short-lived and runs as non-root.
+		unit.NewUnitOption("Service", "SyslogIdentifier", appName.String()),
 	}
 
+	// Restrict access to sensitive paths (eg. procfs)
+	opts = protectSystemFiles(opts, appName)
+
+	if ra.ReadOnlyRootFS {
+		opts = append(opts, unit.NewUnitOption("Service", "ReadOnlyDirectories", common.RelAppRootfsPath(appName)))
+	}
+
+	// TODO(tmrts): Extract this logic into a utility function.
+	vols := make(map[types.ACName]types.Volume)
+	for _, v := range p.Manifest.Volumes {
+		vols[v.Name] = v
+	}
+
+	absRoot, err := filepath.Abs(p.Root) // Absolute path to the pod's rootfs.
+	if err != nil {
+		return err
+	}
+	appRootfs := common.AppRootfsPath(absRoot, appName)
+
+	rwDirs := []string{}
+	imageManifest := p.Images[appName.String()]
+	for _, m := range GenerateMounts(ra, vols, imageManifest) {
+		mntPath, err := EvaluateSymlinksInsideApp(appRootfs, m.Path)
+		if err != nil {
+			return err
+		}
+
+		if !IsMountReadOnly(vols[m.Volume], app.MountPoints) {
+			rwDirs = append(rwDirs, filepath.Join(common.RelAppRootfsPath(appName), mntPath))
+		}
+	}
+
+	opts = append(opts, unit.NewUnitOption("Service", "ReadWriteDirectories", strings.Join(rwDirs, " ")))
+
 	if interactive {
 		opts = append(opts, unit.NewUnitOption("Service", "StandardInput", "tty"))
 		opts = append(opts, unit.NewUnitOption("Service", "StandardOutput", "tty"))
@@ -475,7 +498,6 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 	} else {
 		opts = append(opts, unit.NewUnitOption("Service", "StandardOutput", "journal+console"))
 		opts = append(opts, unit.NewUnitOption("Service", "StandardError", "journal+console"))
-		opts = append(opts, unit.NewUnitOption("Service", "SyslogIdentifier", filepath.Base(app.Exec[0])))
 	}
 
 	// When an app fails, we shut down the pod
@@ -603,66 +625,48 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 //   3. a number
 //   4. a name in reference to /etc/{group,passwd} in the image
 // See https://github.com/appc/spec/blob/master/spec/aci.md#image-manifest-schema
-func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *uid.UidRange) (int, int, error) {
-	app := ra.App
-	appName := ra.Name
-
-	var uid_, gid_ int
+func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *user.UidRange) (int, int, error) {
+	var uidResolver, gidResolver user.Resolver
+	var uid, gid int
 	var err error
 
-	switch {
-	case app.User == "root":
-		uid_ = 0
-	case strings.HasPrefix(app.User, "/"):
-		var stat syscall.Stat_t
-		if err = syscall.Lstat(filepath.Join(common.AppRootfsPath(p.Root, appName),
-			app.User), &stat); err != nil {
-			return -1, -1, errwrap.Wrap(fmt.Errorf("unable to get uid from file %q",
-				app.User), err)
-		}
-		uidReal, _, err := uidRange.UnshiftRange(stat.Uid, 0)
-		if err != nil {
-			return -1, -1, errwrap.Wrap(errors.New("unable to determine real uid"), err)
-		}
-		uid_ = int(uidReal)
-	default:
-		uid_, err = strconv.Atoi(app.User)
-		if err != nil {
-			uid_, err = passwd.LookupUidFromFile(app.User,
-				filepath.Join(common.AppRootfsPath(p.Root, appName), "etc/passwd"))
-			if err != nil {
-				return -1, -1, errwrap.Wrap(fmt.Errorf("cannot lookup user %q", app.User), err)
-			}
-		}
+	root := common.AppRootfsPath(p.Root, ra.Name)
+
+	uidResolver, err = user.NumericIDs(ra.App.User)
+	if err != nil {
+		uidResolver, err = user.IDsFromStat(root, ra.App.User, uidRange)
 	}
 
-	switch {
-	case app.Group == "root":
-		gid_ = 0
-	case strings.HasPrefix(app.Group, "/"):
-		var stat syscall.Stat_t
-		if err = syscall.Lstat(filepath.Join(common.AppRootfsPath(p.Root, appName),
-			app.Group), &stat); err != nil {
-			return -1, -1, errwrap.Wrap(fmt.Errorf("unable to get gid from file %q",
-				app.Group), err)
-		}
-		_, gidReal, err := uidRange.UnshiftRange(0, stat.Gid)
-		if err != nil {
-			return -1, -1, errwrap.Wrap(errors.New("unable to determine real gid"), err)
-		}
-		gid_ = int(gidReal)
-	default:
-		gid_, err = strconv.Atoi(app.Group)
-		if err != nil {
-			gid_, err = group.LookupGidFromFile(app.Group,
-				filepath.Join(common.AppRootfsPath(p.Root, appName), "etc/group"))
-			if err != nil {
-				return -1, -1, errwrap.Wrap(fmt.Errorf("cannot lookup group %q", app.Group), err)
-			}
-		}
+	if err != nil {
+		uidResolver, err = user.IDsFromEtc(root, ra.App.User, "")
 	}
 
-	return uid_, gid_, nil
+	if err != nil { // give up
+		return -1, -1, errwrap.Wrap(fmt.Errorf("invalid user %q", ra.App.User), err)
+	}
+
+	if uid, _, err = uidResolver.IDs(); err != nil {
+		return -1, -1, errwrap.Wrap(fmt.Errorf("failed to configure user %q", ra.App.User), err)
+	}
+
+	gidResolver, err = user.NumericIDs(ra.App.Group)
+	if err != nil {
+		gidResolver, err = user.IDsFromStat(root, ra.App.Group, uidRange)
+	}
+
+	if err != nil {
+		gidResolver, err = user.IDsFromEtc(root, "", ra.App.Group)
+	}
+
+	if err != nil { // give up
+		return -1, -1, errwrap.Wrap(fmt.Errorf("invalid group %q", ra.App.Group), err)
+	}
+
+	if _, gid, err = gidResolver.IDs(); err != nil {
+		return -1, -1, errwrap.Wrap(fmt.Errorf("failed to configure group %q", ra.App.Group), err)
+	}
+
+	return uid, gid, nil
 }
 
 func writeShutdownService(p *stage1commontypes.Pod) error {
@@ -713,7 +717,7 @@ func writeShutdownService(p *stage1commontypes.Pod) error {
 // writeEnvFile creates an environment file for given app name, the minimum
 // required environment variables by the appc spec will be set to sensible
 // defaults here if they're not provided by env.
-func writeEnvFile(p *stage1commontypes.Pod, env types.Environment, appName types.ACName, uidRange *uid.UidRange, separator byte, envFilePath string) error {
+func writeEnvFile(p *stage1commontypes.Pod, env types.Environment, appName types.ACName, uidRange *user.UidRange, separator byte, envFilePath string) error {
 	ef := bytes.Buffer{}
 
 	for dk, dv := range defaultEnv {
@@ -871,7 +875,11 @@ func appToNspawnArgs(p *stage1commontypes.Pod, ra *schema.RuntimeApp) ([]string,
 		args = append(args, strings.Join(opt, ""))
 	}
 
-	capList := strings.Join(GetAppCapabilities(app.Isolators), ",")
+	capabilitiesStr, err := getAppCapabilities(app.Isolators)
+	if err != nil {
+		return nil, err
+	}
+	capList := strings.Join(capabilitiesStr, ",")
 	args = append(args, "--capability="+capList)
 
 	return args, nil
@@ -959,18 +967,81 @@ func GetMachineID(p *stage1commontypes.Pod) string {
 	return "rkt-" + p.UUID.String()
 }
 
-// GetAppCapabilities is a filter which returns a string slice of valid Linux capabilities
-// It requires list of available isolators
-func GetAppCapabilities(isolators types.Isolators) []string {
-	var caps []string
+// getAppCapabilities computes the set of Linux capabilities that an app
+// should have based on its isolators. Only the following capabalities matter:
+// - os/linux/capabilities-retain-set
+// - os/linux/capabilities-remove-set
+//
+// The resulting capabilities are generated following the rules from the spec:
+// See: https://github.com/appc/spec/blob/master/spec/ace.md#linux-isolators
+func getAppCapabilities(isolators types.Isolators) ([]string, error) {
+	var capsToRetain []string
+	var capsToRemove []string
 
+	// Default caps defined in
+	// https://github.com/appc/spec/blob/master/spec/ace.md#linux-isolators
+	appDefaultCapabilities := []string{
+		"CAP_AUDIT_WRITE",
+		"CAP_CHOWN",
+		"CAP_DAC_OVERRIDE",
+		"CAP_FSETID",
+		"CAP_FOWNER",
+		"CAP_KILL",
+		"CAP_MKNOD",
+		"CAP_NET_RAW",
+		"CAP_NET_BIND_SERVICE",
+		"CAP_SETUID",
+		"CAP_SETGID",
+		"CAP_SETPCAP",
+		"CAP_SETFCAP",
+		"CAP_SYS_CHROOT",
+	}
+
+	// Iterate over the isolators defined in
+	// https://github.com/appc/spec/blob/master/spec/ace.md#linux-isolators
+	// Only read the capababilities isolators:
+	// - os/linux/capabilities-retain-set
+	// - os/linux/capabilities-remove-set
 	for _, isolator := range isolators {
-		if capSet, ok := isolator.Value().(types.LinuxCapabilitiesSet); ok &&
-			isolator.Name == types.LinuxCapabilitiesRetainSetName {
-			caps = append(caps, parseLinuxCapabilitiesSet(capSet)...)
+		if capSet, ok := isolator.Value().(types.LinuxCapabilitiesSet); ok {
+			switch isolator.Name {
+			case types.LinuxCapabilitiesRetainSetName:
+				capsToRetain = append(capsToRetain, parseLinuxCapabilitiesSet(capSet)...)
+			case types.LinuxCapabilitiesRevokeSetName:
+				capsToRemove = append(capsToRemove, parseLinuxCapabilitiesSet(capSet)...)
+			}
 		}
 	}
-	return caps
+
+	// appc/spec does not allow to have both the retain set and the remove
+	// set defined.
+	if len(capsToRetain) > 0 && len(capsToRemove) > 0 {
+		return nil, errors.New("cannot have both os/linux/capabilities-retain-set and os/linux/capabilities-remove-set")
+	}
+
+	// Neither the retain set or the remove set are defined
+	if len(capsToRetain) == 0 && len(capsToRemove) == 0 {
+		return appDefaultCapabilities, nil
+	}
+
+	if len(capsToRetain) > 0 {
+		return capsToRetain, nil
+	}
+
+	if len(capsToRemove) == 0 {
+		panic("len(capsToRetain) is negative. This cannot happen.")
+	}
+
+	caps := appDefaultCapabilities
+	for _, rc := range capsToRemove {
+		// backward loop to be safe against deletion
+		for i := len(caps) - 1; i >= 0; i-- {
+			if caps[i] == rc {
+				caps = append(caps[:i], caps[i+1:]...)
+			}
+		}
+	}
+	return caps, nil
 }
 
 // parseLinuxCapabilitySet parses a LinuxCapabilitiesSet into string slice
@@ -981,3 +1052,56 @@ func parseLinuxCapabilitiesSet(capSet types.LinuxCapabilitiesSet) []string {
 	}
 	return capsStr
 }
+
+func getAppNoNewPrivileges(isolators types.Isolators) bool {
+	for _, isolator := range isolators {
+		noNewPrivileges, ok := isolator.Value().(*types.LinuxNoNewPrivileges)
+
+		if ok && bool(*noNewPrivileges) {
+			return true
+		}
+	}
+
+	return false
+}
+
+// restrictProcFS restricts access to some security-sensitive paths under
+// /proc and /sys. Entries are either hidden or just made read-only to app.
+func protectSystemFiles(opts []*unit.UnitOption, appName types.ACName) []*unit.UnitOption {
+	roPaths := []string{
+		"/proc/bus/",
+		"/proc/sys/kernel/core_pattern",
+		"/proc/sys/kernel/modprobe",
+		"/proc/sys/vm/panic_on_oom",
+		"/proc/sysrq-trigger",
+		"/sys/block/",
+		"/sys/bus/",
+		"/sys/class/",
+		"/sys/dev/",
+		"/sys/devices/",
+		"/sys/kernel/",
+	}
+	hiddenPaths := []string{
+		// TODO(lucab): file-paths restrictions need support in systemd first
+		//"/proc/config.gz",
+		//"/proc/kallsyms",
+		//"/proc/sched_debug",
+		//"/proc/kcore",
+		//"/proc/kmem",
+		//"/proc/mem",
+		"/sys/firmware/",
+		"/sys/fs/",
+		"/sys/hypervisor/",
+		"/sys/module/",
+		"/sys/power/",
+	}
+	// Paths prefixed with "-" are ignored if they do not exist:
+	// https://www.freedesktop.org/software/systemd/man/systemd.exec.html#ReadWriteDirectories=
+	for _, p := range hiddenPaths {
+		opts = append(opts, unit.NewUnitOption("Service", "InaccessibleDirectories", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
+	}
+	for _, p := range roPaths {
+		opts = append(opts, unit.NewUnitOption("Service", "ReadOnlyDirectories", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
+	}
+	return opts
+}
