@@ -36,6 +36,7 @@ import (
 	"github.com/appc/goaci/proj2aci"
 	"github.com/appc/spec/schema/types"
 	"github.com/coreos/go-systemd/util"
+	"github.com/coreos/pkg/dlopen"
 	"github.com/godbus/dbus"
 	"github.com/godbus/dbus/introspect"
 	"github.com/hashicorp/errwrap"
@@ -382,22 +383,23 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, debug bool, n *networki
 		return nil, nil, fmt.Errorf("unrecognized stage1 flavor: %q", flavor)
 	}
 
+	// systemd-nspawn needs /etc/machine-id to link the container's journal
+	// to the host. Since systemd-v230, /etc/machine-id is mandatory, see
+	// https://github.com/systemd/systemd/commit/e01ff70a77e781734e1e73a2238af2e9bf7967a8
+	mPath := filepath.Join(common.Stage1RootfsPath(p.Root), "etc", "machine-id")
+	mID := strings.Replace(p.UUID.String(), "-", "", -1)
+
+	if err := ioutil.WriteFile(mPath, []byte(mID), 0644); err != nil {
+		log.FatalE("error writing /etc/machine-id", err)
+	}
+
 	// link journal only if the host is running systemd
 	if util.IsRunningSystemd() {
-		// we write /etc/machine-id here because systemd-nspawn needs it to link
-		// the container's journal to the host
-		mPath := filepath.Join(common.Stage1RootfsPath(p.Root), "etc", "machine-id")
-		mID := strings.Replace(p.UUID.String(), "-", "", -1)
-
-		if err := ioutil.WriteFile(mPath, []byte(mID), 0644); err != nil {
-			log.FatalE("error writing /etc/machine-id", err)
-		}
-
 		args = append(args, "--link-journal=try-guest")
 
 		keepUnit, err := util.RunningFromSystemService()
 		if err != nil {
-			if err == util.ErrSoNotFound {
+			if err == dlopen.ErrSoNotFound {
 				log.Print("warning: libsystemd not found even though systemd is running. Cgroup limits set by the environment (e.g. a systemd service) won't be enforced.")
 			} else {
 				return nil, nil, errwrap.Wrap(errors.New("error determining if we're running from a system service"), err)
@@ -407,6 +409,8 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, debug bool, n *networki
 		if keepUnit {
 			args = append(args, "--keep-unit")
 		}
+	} else {
+		args = append(args, "--link-journal=no")
 	}
 
 	if !debug {
@@ -532,7 +536,7 @@ func stage1() int {
 		}
 
 		if len(mdsToken) > 0 {
-			hostIP, err := n.GetDefaultHostIP()
+			hostIP, err := n.GetForwardableNetHostIP()
 			if err != nil {
 				log.PrintE("failed to get default Host IP", err)
 				return 1
@@ -565,6 +569,7 @@ func stage1() int {
 	}
 
 	if flavor == "kvm" {
+		kvm.InitDebug(debug)
 		if err := KvmNetworkingToSystemd(p, n); err != nil {
 			log.PrintE("failed to configure systemd for kvm", err)
 			return 1
@@ -619,6 +624,11 @@ func stage1() int {
 	machineID := stage1initcommon.GetMachineID(p)
 	subcgroup, err := getContainerSubCgroup(machineID)
 	if err == nil {
+		if err := ioutil.WriteFile(filepath.Join(p.Root, "subcgroup"),
+			[]byte(fmt.Sprintf("%s", subcgroup)), 0644); err != nil {
+			log.FatalE("cannot write subcgroup file", err)
+			return 1
+		}
 		if err := mountContainerCgroups(s1Root, enabledCgroups, subcgroup, serviceNames); err != nil {
 			log.PrintE("couldn't mount the container cgroups", err)
 			return 1
@@ -729,7 +739,7 @@ func getContainerSubCgroup(machineID string) (string, error) {
 		if err != nil {
 			return "", errwrap.Wrap(errors.New("could not get unit name"), err)
 		}
-		subcgroup = filepath.Join(slicePath, unit, "system.slice")
+		subcgroup = filepath.Join(slicePath, unit)
 	} else {
 		escapedmID := strings.Replace(machineID, "-", "\\x2d", -1)
 		machineDir := "machine-" + escapedmID + ".scope"
@@ -737,7 +747,7 @@ func getContainerSubCgroup(machineID string) (string, error) {
 			// we are not in the final cgroup yet: systemd-nspawn will move us
 			// to the correct cgroup later during registration so we can't
 			// look it up in /proc/self/cgroup
-			subcgroup = filepath.Join("machine.slice", machineDir, "system.slice")
+			subcgroup = filepath.Join("machine.slice", machineDir)
 		} else {
 			// when registration is disabled the container will be directly
 			// under the current cgroup so we can look it up in /proc/self/cgroup
@@ -748,11 +758,10 @@ func getContainerSubCgroup(machineID string) (string, error) {
 			// systemd-nspawn won't work if we are in the root cgroup. In addition,
 			// we want all rkt instances to be in distinct cgroups. Create a
 			// subcgroup and add ourselves to it.
-			ownCgroupPath = filepath.Join(ownCgroupPath, machineDir)
-			if err := cgroup.JoinSubcgroup("systemd", ownCgroupPath); err != nil {
-				return "", errwrap.Wrap(fmt.Errorf("error joining %s subcgroup", ownCgroupPath), err)
+			subcgroup = filepath.Join(ownCgroupPath, machineDir)
+			if err := cgroup.JoinSubcgroup("systemd", subcgroup); err != nil {
+				return "", errwrap.Wrap(fmt.Errorf("error joining %s subcgroup", subcgroup), err)
 			}
-			subcgroup = filepath.Join(ownCgroupPath, "system.slice")
 		}
 	}
 
