@@ -18,9 +18,9 @@ package main
 
 import "testing"
 
-func TestNetDefaultPortFwdConnectivity(t *testing.T) {
-	NewNetDefaultPortFwdConnectivityTest(
-		PortFwdCase{"172.16.28.1", "--net=default", true},
+func TestNetPortFwdConnectivity(t *testing.T) {
+	NewNetPortFwdConnectivityTest(
+		defaultSamePortFwdCase,
 	).Execute(t)
 }
 
@@ -28,3 +28,15 @@ func TestNetCustomPtp(t *testing.T) {
 	// PTP means connection Point-To-Point. That is, connections to other pods/containers should be forbidden
 	NewNetCustomPtpTest(false)
 }
+
+func TestNetDefaultNetNS(t *testing.T) {
+	NewTestNetDefaultNetNS().Execute(t)
+}
+
+func TestNetLongName(t *testing.T) {
+	NewTestNetLongName().Execute(t)
+}
+
+func TestNetDefaultRestrictedConnectivity(t *testing.T) {
+	NewTestNetDefaultRestrictedConnectivity().Execute(t)
+}
