@@ -175,37 +175,39 @@ func NewNetNoneTest() testutils.Test {
  * ---
  * Container must be in a separate network namespace
  */
-func TestNetDefaultNetNS(t *testing.T) {
-	testImageArgs := []string{"--exec=/inspect --print-netns"}
-	testImage := patchTestACI("rkt-inspect-networking.aci", testImageArgs...)
-	defer os.Remove(testImage)
+func NewTestNetDefaultNetNS() testutils.Test {
+	return testutils.TestFunc(func(t *testing.T) {
+		testImageArgs := []string{"--exec=/inspect --print-netns"}
+		testImage := patchTestACI("rkt-inspect-networking.aci", testImageArgs...)
+		defer os.Remove(testImage)
 
-	ctx := testutils.NewRktRunCtx()
-	defer ctx.Cleanup()
+		ctx := testutils.NewRktRunCtx()
+		defer ctx.Cleanup()
 
-	f := func(argument string) {
-		cmd := fmt.Sprintf("%s --debug --insecure-options=image run %s --mds-register=false %s", ctx.Cmd(), argument, testImage)
-		child := spawnOrFail(t, cmd)
-		defer waitOrFail(t, child, 0)
+		f := func(argument string) {
+			cmd := fmt.Sprintf("%s --debug --insecure-options=image run %s --mds-register=false %s", ctx.Cmd(), argument, testImage)
+			child := spawnOrFail(t, cmd)
+			defer waitOrFail(t, child, 0)
+
+			expectedRegex := `NetNS: (net:\[\d+\])`
+			result, out, err := expectRegexWithOutput(child, expectedRegex)
+			if err != nil {
+				t.Fatalf("Error: %v\nOutput: %v", err, out)
+			}
+
+			ns, err := os.Readlink("/proc/self/ns/net")
+			if err != nil {
+				t.Fatalf("Cannot evaluate NetNS symlink: %v", err)
+			}
+
+			if nsChanged := ns != result[1]; !nsChanged {
+				t.Fatalf("container did not leave host netns")
+			}
 
-		expectedRegex := `NetNS: (net:\[\d+\])`
-		result, out, err := expectRegexWithOutput(child, expectedRegex)
-		if err != nil {
-			t.Fatalf("Error: %v\nOutput: %v", err, out)
 		}
-
-		ns, err := os.Readlink("/proc/self/ns/net")
-		if err != nil {
-			t.Fatalf("Cannot evaluate NetNS symlink: %v", err)
-		}
-
-		if nsChanged := ns != result[1]; !nsChanged {
-			t.Fatalf("container did not leave host netns")
-		}
-
-	}
-	f("--net=default")
-	f("")
+		f("--net=default")
+		f("")
+	})
 }
 
 /*
@@ -293,81 +295,119 @@ func NewNetDefaultConnectivityTest() testutils.Test {
  * eth0's IPv4
  * TODO: verify that the container isn't NATed
  */
-func TestNetDefaultRestrictedConnectivity(t *testing.T) {
-	ctx := testutils.NewRktRunCtx()
-	defer ctx.Cleanup()
+func NewTestNetDefaultRestrictedConnectivity() testutils.Test {
+	return testutils.TestFunc(func(t *testing.T) {
+		ctx := testutils.NewRktRunCtx()
+		defer ctx.Cleanup()
 
-	f := func(argument string) {
-		httpPort, err := testutils.GetNextFreePort4()
-		if err != nil {
-			t.Fatalf("%v", err)
-		}
-		httpServeAddr := fmt.Sprintf("0.0.0.0:%v", httpPort)
-		iface := "eth0"
-
-		testImageArgs := []string{fmt.Sprintf("--exec=/inspect --print-ipv4=%v --serve-http=%v", iface, httpServeAddr)}
-		testImage := patchTestACI("rkt-inspect-networking.aci", testImageArgs...)
-		defer os.Remove(testImage)
-
-		cmd := fmt.Sprintf("%s --debug --insecure-options=image run %s --mds-register=false %s", ctx.Cmd(), argument, testImage)
-		child := spawnOrFail(t, cmd)
-
-		expectedRegex := `IPv4: (\d+\.\d+\.\d+\.\d+)`
-		result, out, err := expectRegexWithOutput(child, expectedRegex)
-		if err != nil {
-			t.Fatalf("Error: %v\nOutput: %v", err, out)
-		}
-		httpGetAddr := fmt.Sprintf("http://%v:%v", result[1], httpPort)
-
-		ga := testutils.NewGoroutineAssistant(t)
-		ga.Add(2)
-
-		// Child opens the server
-		go func() {
-			defer ga.Done()
-			ga.WaitOrFail(child)
-		}()
-
-		// Host connects to the child
-		go func() {
-			defer ga.Done()
-			expectedRegex := `serving on`
-			_, out, err := expectRegexWithOutput(child, expectedRegex)
+		f := func(argument string) {
+			httpPort, err := testutils.GetNextFreePort4()
 			if err != nil {
-				ga.Fatalf("Error: %v\nOutput: %v", err, out)
+				t.Fatalf("%v", err)
 			}
-			body, err := testutils.HTTPGet(httpGetAddr)
-			if err != nil {
-				ga.Fatalf("%v\n", err)
-			}
-			t.Logf("HTTP-Get received: %s", body)
-		}()
+			httpServeAddr := fmt.Sprintf("0.0.0.0:%v", httpPort)
+			iface := "eth0"
 
-		ga.Wait()
-	}
-	f("--net=default-restricted")
+			testImageArgs := []string{fmt.Sprintf("--exec=/inspect --print-ipv4=%v --serve-http=%v", iface, httpServeAddr)}
+			testImage := patchTestACI("rkt-inspect-networking.aci", testImageArgs...)
+			defer os.Remove(testImage)
+
+			cmd := fmt.Sprintf("%s --debug --insecure-options=image run %s --mds-register=false %s", ctx.Cmd(), argument, testImage)
+			child := spawnOrFail(t, cmd)
+
+			expectedRegex := `IPv4: (\d+\.\d+\.\d+\.\d+)`
+			result, out, err := expectRegexWithOutput(child, expectedRegex)
+			if err != nil {
+				t.Fatalf("Error: %v\nOutput: %v", err, out)
+			}
+			httpGetAddr := fmt.Sprintf("http://%v:%v", result[1], httpPort)
+
+			ga := testutils.NewGoroutineAssistant(t)
+			ga.Add(2)
+
+			// Child opens the server
+			go func() {
+				defer ga.Done()
+				ga.WaitOrFail(child)
+			}()
+
+			// Host connects to the child
+			go func() {
+				defer ga.Done()
+				expectedRegex := `serving on`
+				_, out, err := expectRegexWithOutput(child, expectedRegex)
+				if err != nil {
+					ga.Fatalf("Error: %v\nOutput: %v", err, out)
+				}
+				body, err := testutils.HTTPGet(httpGetAddr)
+				if err != nil {
+					ga.Fatalf("%v\n", err)
+				}
+				t.Logf("HTTP-Get received: %s", body)
+			}()
+
+			ga.Wait()
+		}
+		f("--net=default-restricted")
+	})
 }
 
 type PortFwdCase struct {
 	HttpGetIP     string
+	HttpServePort int
 	RktArg        string
 	ShouldSucceed bool
 }
 
+var (
+	bannedPorts = make(map[int]struct{}, 0)
+
+	defaultSamePortFwdCase   = PortFwdCase{"172.16.28.1", 0, "--net=default", true}
+	defaultDiffPortFwdCase   = PortFwdCase{"172.16.28.1", 1024, "--net=default", true}
+	defaultLoSamePortFwdCase = PortFwdCase{"127.0.0.1", 0, "--net=default", true}
+	defaultLoDiffPortFwdCase = PortFwdCase{"127.0.0.1", 1014, "--net=default", true}
+
+	portFwdBridge = networkTemplateT{
+		Name:      "bridge1",
+		Type:      "bridge",
+		Bridge:    "bridge1",
+		IpMasq:    true,
+		IsGateway: true,
+		Ipam: ipamTemplateT{
+			Type:   "host-local",
+			Subnet: "11.11.5.0/24",
+			Routes: []map[string]string{
+				{"dst": "0.0.0.0/0"},
+			},
+		},
+	}
+	bridgeSamePortFwdCase   = PortFwdCase{"11.11.5.1", 0, "--net=" + portFwdBridge.Name, true}
+	bridgeDiffPortFwdCase   = PortFwdCase{"11.11.5.1", 1024, "--net=" + portFwdBridge.Name, true}
+	bridgeLoSamePortFwdCase = PortFwdCase{"127.0.0.1", 0, "--net=" + portFwdBridge.Name, true}
+	bridgeLoDiffPortFwdCase = PortFwdCase{"127.0.0.1", 1024, "--net=" + portFwdBridge.Name, true}
+)
+
 func (ct PortFwdCase) Execute(t *testing.T, ctx *testutils.RktRunCtx) {
+	netdir := prepareTestNet(t, ctx, portFwdBridge)
+	defer os.RemoveAll(netdir)
 
-	bannedPorts := make(map[int]struct{}, 0)
 	httpPort, err := testutils.GetNextFreePort4Banned(bannedPorts)
 	if err != nil {
 		t.Fatalf("%v", err)
 	}
 	bannedPorts[httpPort] = struct{}{}
 
-	httpServeAddr := fmt.Sprintf("0.0.0.0:%d", httpPort)
+	httpServePort := ct.HttpServePort
+	if httpServePort == 0 {
+		httpServePort = httpPort
+	}
+
+	httpServeAddr := fmt.Sprintf("0.0.0.0:%d", httpServePort)
 	testImageArgs := []string{
-		fmt.Sprintf("--ports=http,protocol=tcp,port=%d", httpPort),
+		fmt.Sprintf("--ports=http,protocol=tcp,port=%d", httpServePort),
 		fmt.Sprintf("--exec=/inspect --serve-http=%v", httpServeAddr),
 	}
+	t.Logf("testImageArgs: %v", testImageArgs)
 	testImage := patchTestACI("rkt-inspect-networking.aci", testImageArgs...)
 	defer os.Remove(testImage)
 
@@ -410,10 +450,6 @@ func (ct PortFwdCase) Execute(t *testing.T, ctx *testutils.RktRunCtx) {
 	}()
 
 	ga.Wait()
-
-	// TODO: ensure that default-restricted is not accessible from non-host
-	// f("172.16.28.1", "--net=default-restricted", true)
-	// f("127.0.0.1", "--net=default-restricted", true)
 }
 
 type portFwdTest []PortFwdCase
@@ -428,12 +464,12 @@ func (ct portFwdTest) Execute(t *testing.T) {
 }
 
 /*
- * Default net port forwarding connectivity
+ * Net port forwarding connectivity
  * ---
  * Container launches http server on all its interfaces
  * Host must be able to connect to container's http server on it's own interfaces
  */
-func NewNetDefaultPortFwdConnectivityTest(cases ...PortFwdCase) testutils.Test {
+func NewNetPortFwdConnectivityTest(cases ...PortFwdCase) testutils.Test {
 	return portFwdTest(cases)
 }
 
@@ -465,6 +501,7 @@ type networkTemplateT struct {
 	Master    string `json:"master,omitempty"`
 	IpMasq    bool
 	IsGateway bool
+	Bridge    string `json:"bridge,omitempty"`
 	Ipam      ipamTemplateT
 }
 
@@ -784,18 +821,20 @@ func NewNetOverrideTest() testutils.Test {
 	})
 }
 
-func TestNetLongName(t *testing.T) {
-	nt := networkTemplateT{
-		Name:   "thisnameiswaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaytoolong",
-		Type:   "ptp",
-		IpMasq: true,
-		Ipam: ipamTemplateT{
-			Type:   "host-local",
-			Subnet: "11.11.6.0/24",
-			Routes: []map[string]string{
-				{"dst": "0.0.0.0/0"},
+func NewTestNetLongName() testutils.Test {
+	return testutils.TestFunc(func(t *testing.T) {
+		nt := networkTemplateT{
+			Name:   "thisnameiswaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaytoolong",
+			Type:   "ptp",
+			IpMasq: true,
+			Ipam: ipamTemplateT{
+				Type:   "host-local",
+				Subnet: "11.11.6.0/24",
+				Routes: []map[string]string{
+					{"dst": "0.0.0.0/0"},
+				},
 			},
-		},
-	}
-	testNetCustomNatConnectivity(t, nt)
+		}
+		testNetCustomNatConnectivity(t, nt)
+	})
 }
