@@ -119,6 +119,28 @@ func TestPodManifest(t *testing.T) {
 			`'"[$]`,
 			"",
 		},
+		{
+			// Working directory.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-working-directory.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:             []string{"/inspect", "--print-cwd"},
+							User:             "0",
+							Group:            "0",
+							WorkingDirectory: "/dir1",
+						},
+					},
+				},
+			},
+			0,
+			"cwd: /dir1",
+			"",
+		},
 		{
 			// Simple read.
 			[]imagePatch{
@@ -143,6 +165,31 @@ func TestPodManifest(t *testing.T) {
 			"dir1",
 			"",
 		},
+		{
+			// Simple read from read-only rootfs.
+			[]imagePatch{
+				{"rkt-test-run-read-only-rootfs-pod-manifest-read.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1/file"},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+			},
+			0,
+			"dir1",
+			"",
+		},
 		{
 			// Simple read after write with *empty* volume mounted.
 			[]imagePatch{
@@ -180,6 +227,44 @@ func TestPodManifest(t *testing.T) {
 			"empty:foo",
 			"",
 		},
+		{
+			// Simple read from read-only rootfs after write with *empty* volume mounted.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-read-only-rootfs-empty-vol-rw.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--write-file", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1/file"},
+								{"CONTENT", "empty:foo"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir1", false},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{
+						Name: "dir1",
+						Kind: "empty",
+						Mode: stringP("0755"),
+						UID:  intP(0),
+						GID:  intP(0),
+					},
+				},
+			},
+			0,
+			"empty:foo",
+			"",
+		},
 		{
 			// Stat directory in a *empty* volume mounted.
 			[]imagePatch{
@@ -216,6 +301,43 @@ func TestPodManifest(t *testing.T) {
 			"(?s)/dir1: mode: d--x-w--wx.*" + "/dir1: user: 9991.*" + "/dir1: group: 9992",
 			"",
 		},
+		{
+			// Stat directory in a *empty* volume mounted using a read-only rootfs.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-read-only-rootfs-empty-vol-stat.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--stat-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir1", false},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{
+						Name: "dir1",
+						Kind: "empty",
+						Mode: stringP("0123"),
+						UID:  intP(9991),
+						GID:  intP(9992),
+					},
+				},
+			},
+			0,
+			"(?s)/dir1: mode: d--x-w--wx.*" + "/dir1: user: 9991.*" + "/dir1: group: 9992",
+			"",
+		},
 		{
 			// Simple read after write with volume mounted.
 			[]imagePatch{
@@ -247,6 +369,38 @@ func TestPodManifest(t *testing.T) {
 			"host:foo",
 			"",
 		},
+		{
+			// Simple read after write with volume mounted in a read-only rootfs.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-read-only-rootfs-vol-rw.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--write-file", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1/file"},
+								{"CONTENT", "host:foo"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir1", false},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+				},
+			},
+			0,
+			"host:foo",
+			"",
+		},
 		{
 			// Simple read after write with read-only mount point, should fail.
 			[]imagePatch{
@@ -278,6 +432,38 @@ func TestPodManifest(t *testing.T) {
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
+		{
+			// Simple read after write with read-only mount point in a read-only rootfs, should fail.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-read-only-rootfs-vol-ro.aci", []string{}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--write-file", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1/file"},
+								{"CONTENT", "bar"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir1", true},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+				},
+			},
+			1,
+			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
+			"",
+		},
 		{
 			// Simple read after write with volume mounted.
 			// Override the image's mount point spec. This should fail as the volume is
@@ -311,6 +497,40 @@ func TestPodManifest(t *testing.T) {
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
+		{
+			// Simple read after write with volume mounted in a read-only rootfs.
+			// Override the image's mount point spec. This should fail as the volume is
+			// read-only in pod manifest, (which will override the mount point in both image/pod manifest).
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-vol-rw-override.aci", []string{"--mounts=dir1,path=/dir1,readOnly=false"}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--write-file", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir1/file"},
+								{"CONTENT", "bar"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir1", false},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+				},
+			},
+			1,
+			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
+			"",
+		},
 		{
 			// Simple read after write with volume mounted.
 			// Override the image's mount point spec.
@@ -343,13 +563,46 @@ func TestPodManifest(t *testing.T) {
 			"host:bar",
 			"",
 		},
+		{
+			// Simple read after write with volume mounted in a read-only rootfs.
+			// Override the image's mount point spec.
+			[]imagePatch{
+				{"rkt-test-run-pod-manifest-read-only-rootfs-vol-rw-override.aci", []string{"--mounts=dir1,path=/dir1,readOnly=true"}},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name: baseAppName,
+						App: &types.App{
+							Exec:  []string{"/inspect", "--write-file", "--read-file"},
+							User:  "0",
+							Group: "0",
+							Environment: []types.EnvironmentVariable{
+								{"FILE", "/dir2/file"},
+								{"CONTENT", "host:bar"},
+							},
+							MountPoints: []types.MountPoint{
+								{"dir1", "/dir2", false},
+							},
+						},
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+				},
+			},
+			0,
+			"host:bar",
+			"",
+		},
 		{
 			// Simple read after write with volume mounted, no apps in pod manifest.
 			[]imagePatch{
 				{
 					"rkt-test-run-pod-manifest-vol-rw-no-app.aci",
 					[]string{
-						"--exec=/inspect --write-file --read-file --file-name=/dir1/file --content=host:baz",
+						"--exec=/inspect --write-file --read-file --file-name=/dir1/file --content=host:baw",
 						"--mounts=dir1,path=/dir1,readOnly=false",
 					},
 				},
@@ -363,13 +616,39 @@ func TestPodManifest(t *testing.T) {
 				},
 			},
 			0,
+			"host:baw",
+			"",
+		},
+		{
+			// Simple read after write with volume mounted in a read-only rootfs, no apps in pod manifest.
+			[]imagePatch{
+				{
+					"rkt-test-run-pod-manifest-read-only-rootfs-vol-rw-no-app.aci",
+					[]string{
+						"--exec=/inspect --write-file --read-file --file-name=/dir1/file --content=host:baz",
+						"--mounts=dir1,path=/dir1,readOnly=false",
+					},
+				},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name:           baseAppName,
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+				},
+			},
+			0,
 			"host:baz",
 			"",
 		},
 		{
 			// Simple read after write with volume mounted, no apps in pod manifest.
 			// This should succeed even the mount point in image manifest is readOnly,
-			// because it is overrided by the volume's readOnly.
+			// because it is overriden by the volume's readOnly.
 			[]imagePatch{
 				{
 					"rkt-test-run-pod-manifest-vol-ro-no-app.aci",
@@ -415,6 +694,33 @@ func TestPodManifest(t *testing.T) {
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
+		{
+			// Simple read after write in read-only rootfs with read-only volume mounted, no apps in pod manifest.
+			// This should fail as the volume is read-only.
+			[]imagePatch{
+				{
+					"rkt-test-run-pod-manifest-read-only-rootfs-vol-ro-no-app.aci",
+					[]string{
+						"--exec=/inspect --write-file --read-file --file-name=/dir1/file --content=baz",
+						"--mounts=dir1,path=/dir1,readOnly=false",
+					},
+				},
+			},
+			&schema.PodManifest{
+				Apps: []schema.RuntimeApp{
+					{
+						Name:           baseAppName,
+						ReadOnlyRootFS: true,
+					},
+				},
+				Volumes: []types.Volume{
+					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+				},
+			},
+			1,
+			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
+			"",
+		},
 		{
 			// Print CPU quota, which should be overwritten by the pod manifest.
 			[]imagePatch{
@@ -784,7 +1090,8 @@ func TestPodManifest(t *testing.T) {
 
 		if tt.expectedResult != "" {
 			if _, out, err := expectRegexWithOutput(child, tt.expectedResult); err != nil {
-				t.Fatalf("Expected %q but not found: %v\n%s", tt.expectedResult, err, out)
+				t.Errorf("Expected %q but not found: %v\n%s", tt.expectedResult, err, out)
+				continue
 			}
 		}
 		waitOrFail(t, child, tt.expectedExit)
@@ -801,7 +1108,8 @@ func TestPodManifest(t *testing.T) {
 
 		if tt.expectedResult != "" {
 			if _, out, err := expectRegexWithOutput(child, tt.expectedResult); err != nil {
-				t.Fatalf("Expected %q but not found: %v\n%s", tt.expectedResult, err, out)
+				t.Errorf("Expected %q but not found: %v\n%s", tt.expectedResult, err, out)
+				continue
 			}
 		}
 
