@@ -225,6 +225,16 @@ func importImageAndFetchHash(t *testing.T, ctx *testutils.RktRunCtx, fetchArgs s
 	return importImageAndFetchHashAsGid(t, ctx, fetchArgs, img, 0)
 }
 
+func importImageAndRun(imagePath string, t *testing.T, ctx *testutils.RktRunCtx) {
+	cmd := fmt.Sprintf("%s --insecure-options=image run %s", ctx.Cmd(), imagePath)
+	spawnAndWaitOrFail(t, cmd, 0)
+}
+
+func importImageAndPrepare(imagePath string, t *testing.T, ctx *testutils.RktRunCtx) {
+	cmd := fmt.Sprintf("%s --insecure-options=image prepare %s", ctx.Cmd(), imagePath)
+	spawnAndWaitOrFail(t, cmd, 0)
+}
+
 func patchImportAndFetchHash(image string, patches []string, t *testing.T, ctx *testutils.RktRunCtx) string {
 	imagePath := patchTestACI(image, patches...)
 	defer os.Remove(imagePath)
@@ -312,6 +322,31 @@ func runRktAsUidGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, exp
 	}
 }
 
+func runRkt(t *testing.T, rktCmd string, uid, gid int) (string, int) {
+	child, err := gexpect.Command(rktCmd)
+	if err != nil {
+		t.Fatalf("cannot exec rkt: %v", err)
+	}
+	if gid != 0 {
+		child.Cmd.SysProcAttr = &syscall.SysProcAttr{}
+		child.Cmd.SysProcAttr.Credential = &syscall.Credential{Uid: uint32(uid), Gid: uint32(gid)}
+	}
+
+	err = child.Start()
+	if err != nil {
+		t.Fatalf("cannot start rkt: %v", err)
+	}
+
+	_, linesChan := child.AsyncInteractChannels()
+
+	var buf bytes.Buffer
+	for line := range linesChan {
+		buf.WriteString(line + "\n") // reappend newline
+	}
+
+	return buf.String(), getExitStatus(child.Wait())
+}
+
 func startRktAsGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, gid int) *gexpect.ExpectSubprocess {
 	child, err := gexpect.Command(rktCmd)
 	if err != nil {
@@ -334,6 +369,29 @@ func startRktAsGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, gid
 	return child
 }
 
+func startRktAsUidGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, expectError bool, uid, gid int) *gexpect.ExpectSubprocess {
+	child, err := gexpect.Command(rktCmd)
+	if err != nil {
+		t.Fatalf("cannot exec rkt: %v", err)
+	}
+	if gid != 0 {
+		child.Cmd.SysProcAttr = &syscall.SysProcAttr{}
+		child.Cmd.SysProcAttr.Credential = &syscall.Credential{Uid: uint32(uid), Gid: uint32(gid)}
+	}
+
+	err = child.Start()
+	if err != nil {
+		t.Fatalf("cannot start rkt: %v", err)
+	}
+
+	if expectedLine != "" {
+		if err := expectWithOutput(child, expectedLine); err != nil {
+			t.Fatalf("didn't receive expected output %q: %v", expectedLine, err)
+		}
+	}
+	return child
+}
+
 func runRktAndCheckRegexOutput(t *testing.T, rktCmd, match string) {
 	child := spawnOrFail(t, rktCmd)
 	defer child.Wait()
@@ -417,7 +475,7 @@ type podInfo struct {
 	state     string
 	apps      map[string]*appInfo
 	networks  map[string]*networkInfo
-	manifest  []byte
+	manifest  *schema.PodManifest
 	createdAt int64
 	startedAt int64
 }
@@ -435,7 +493,8 @@ func parsePodInfoOutput(t *testing.T, result string, p *podInfo) {
 	for _, line := range lines {
 		tuples := strings.SplitN(line, "=", 2)
 		if len(tuples) != 2 {
-			t.Fatalf("Unexpected line: %v", line)
+			t.Logf("Unexpected line: %v", line)
+			continue
 		}
 
 		switch tuples[0] {
@@ -521,6 +580,7 @@ func getPodDir(t *testing.T, ctx *testutils.RktRunCtx, podID string) string {
 func getPodInfo(t *testing.T, ctx *testutils.RktRunCtx, podID string) *podInfo {
 	p := &podInfo{
 		id:       podID,
+		pid:      -1,
 		apps:     make(map[string]*appInfo),
 		networks: make(map[string]*networkInfo),
 	}
@@ -532,14 +592,13 @@ func getPodInfo(t *testing.T, ctx *testutils.RktRunCtx, podID string) *podInfo {
 	}
 
 	// Trim the last '\n' character.
-	p.manifest = bytes.TrimSpace(output)
+	mfst := bytes.TrimSpace(output)
 
 	// Fill app infos.
-	var manifest schema.PodManifest
-	if err := json.Unmarshal(p.manifest, &manifest); err != nil {
+	if err := json.Unmarshal(mfst, &p.manifest); err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
-	for _, app := range manifest.Apps {
+	for _, app := range p.manifest.Apps {
 		appName := app.Name.String()
 		p.apps[appName] = &appInfo{
 			name: appName,
@@ -549,10 +608,7 @@ func getPodInfo(t *testing.T, ctx *testutils.RktRunCtx, podID string) *podInfo {
 	}
 
 	// Fill other infos.
-	output, err = exec.Command("/bin/bash", "-c", fmt.Sprintf("%s status %s", ctx.Cmd(), podID)).CombinedOutput()
-	if err != nil {
-		t.Fatalf("Unexpected error: %v", err)
-	}
+	output, _ = exec.Command("/bin/bash", "-c", fmt.Sprintf("%s status %s", ctx.Cmd(), podID)).CombinedOutput()
 	parsePodInfoOutput(t, string(output), p)
 
 	return p
@@ -594,7 +650,8 @@ func getImageInfo(t *testing.T, ctx *testutils.RktRunCtx, imageID string) *image
 	imgInfo := parseImageInfoOutput(t, string(output))
 
 	// Get manifest
-	output, err = exec.Command("/bin/bash", "-c", fmt.Sprintf("%s image cat-manifest %s", ctx.Cmd(), imageID)).CombinedOutput()
+	output, err = exec.Command("/bin/bash", "-c",
+		fmt.Sprintf("%s image cat-manifest --pretty-print=false %s", ctx.Cmd(), imageID)).CombinedOutput()
 	if err != nil {
 		t.Fatalf("Unexpected error: %v", err)
 	}
@@ -735,3 +792,26 @@ func checkUserNS() error {
 	// Check if it really works
 	return exec.Command("/bin/bash", "-c", "unshare -U true").Run()
 }
+
+func authDir(confDir string) string {
+	return filepath.Join(confDir, "auth.d")
+}
+
+func pathsDir(confDir string) string {
+	return filepath.Join(confDir, "paths.d")
+}
+
+func stage1Dir(confDir string) string {
+	return filepath.Join(confDir, "stage1.d")
+}
+
+func writeConfig(t *testing.T, dir, filename, contents string) {
+	if err := os.MkdirAll(dir, 0755); err != nil {
+		t.Fatalf("Failed to create config directory %q: %v", dir, err)
+	}
+	path := filepath.Join(dir, filename)
+	os.Remove(path)
+	if err := ioutil.WriteFile(path, []byte(contents), 0644); err != nil {
+		t.Fatalf("Failed to write file %q: %v", path, err)
+	}
+}
